% Compute the element load vectors.
%
% function evs = distrib_loads(self, geom, u, fi)
%
% Compute the element load vectors corresponding to applied distributed
% load. Here it means force per unit length of the beam, 
% in the configuration given by the displacement u.
% Note: the force intensity must be uniform across the entire element.
% 
% Return an array of the load vectors so they may be assembled.
%    Call as
% evs = distrib_loads(feb, geom, u, fi)
%     geom=geometry field
%     u=displacement field
%     fi=force intensity object, must be uniform across each element
%
function evs = distrib_loads(self, geom, u, fi)
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom,'dim');
    % Pre-allocate the element matrices
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(u,conns(i,:),'eqnums');
        Fe{i} =zeros(get(geom,'dim')*nfens,1);
    end
    Xs = gather(geom, (1:get(geom,'nfens')), 'values', 'noreshape'); % reference coordinates
    Us = gather(u, (1:get(geom,'nfens')), 'values', 'noreshape'); % displacement in step n
    % Material
    mat = get(self.feblock, 'mater');
    prop=get(mat, 'property' );
    E=get(prop , 'E' );% Young's modulus
    G=get(prop , 'G' );% shear modulus
    %     matstates = get(self.feblock, 'matstates');
    % Section properties
    A=get(gcells,'A');
    I2=get(gcells,'I2');
    I3=get(gcells,'I3');
    J=get(gcells,'J');
    x1x2_vector=get(gcells,'x1x2_vector');
    % Now loop over all gcells in the block
    Z=zeros(3,3);
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x = Xs(conn,:); % coordinates of nodes
        L0=center_line_length(gcells,x);
        T= local_coordinate_system (gcells, x, x1x2_vector(:,i));
        Te=[T Z Z Z; ...
            Z T Z Z;...
            Z Z T Z;...
            Z Z Z T];
        f=get_magn(fi,0);
        fL =T'*f; % local distributed load components
        FL = [fL(1)*L0/2;
            fL(2)*L0/2;
            fL(3)*L0/2;
            0;
            -fL(3)*L0^2/12;
            +fL(2)*L0^2/12;
            fL(1)*L0/2;
            fL(2)*L0/2;
            fL(3)*L0/2;
            0;
            +fL(3)*L0^2/12;
            -fL(2)*L0^2/12;
            ];
        Fe{i} = Te*FL;
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
