% Compute the element centrifugal load vectors.
%
% function evs = centrifugal_loads(self, geom0, u1, Rfield1, dchi, spin_vector, spin_center)
%
%     geom0=initial geometry field; locations of nodes
%     u1=current geometry increment field; displacements of nodes
%     Rfield1=field of nodal frames, current deformation
%     dchi=field of translation and rotation increments
%             Equation numbers are taken from the field dchi.  
%     spin_vector, spin_center=vector of spin (angular velocity), and 
%           reference point about which the spinning occurs. 
%
% Compute the element load vectors corresponding to  centrifugal distributed
% load. Here it means force per unit length of the beam, 
% in the configuration given by the displacement u.
% 
% Return an array of the load vectors so they may be assembled.
% 
%
function evs = centrifugal_loads_x_only(self, geom0, u1, Rfield1, dchi, spin_velocity, spin_eccentricity)
    spin_vector= [spin_velocity,0,0];
    Omegatilde2=skewmat(spin_vector)*skewmat(spin_vector);
    gcells = get(self.feblock,'gcells');
    ngcells = count(gcells);
    nfens = get(gcells(1),'nfens');
    dim = get(geom0,'dim');
    % Pre-allocate the element matrices
    % Now loop over all gcells in the block
    conns = get(gcells, 'conn'); % connectivity
    Fe = cell(size(conns,1),1);
    eqnums = cell(size(conns,1),1);
    for i=1:size(conns,1)
        eqnums{i} =gather(dchi,conns(i,:),'eqnums');
        Fe{i} =zeros(get(dchi,'dim')*nfens,1);
    end
    Xs =gather(geom0,(1:get(geom0,'nfens')),'values','noreshape');
    U1s =gather(u1,(1:get(u1,'nfens')),'values','noreshape');
    Rs = gather(Rfield1, (1:get(geom0,'nfens')), 'values', 'noreshape'); % displacement in step n
    X1s=Xs+U1s;
    % Material
    mat = get(self.feblock, 'mater');
    prop=get(mat, 'property' );
    E=get(prop , 'E' );% Young's modulus
    G=get(prop , 'G' );% shear modulus
    rho=get(prop , 'rho' );
    %     matstates = get(self.feblock, 'matstates');
    % Section properties
    A=get(gcells,'A');
    I1=get(gcells,'I1');
    I2=get(gcells,'I2');
    I3=get(gcells,'I3');
    J=get(gcells,'J');
    x1x2_vector=get(gcells,'x1x2_vector');
    % Now loop over all gcells in the block
    Z=zeros(3,3);
    for i=1:ngcells
        conn = conns(i,:); % connectivity
        x0 = Xs(conn,:); % coordinates of nodes
        U1 = U1s(conn,:); % coordinates of nodes
        RI =reshape(Rs(conn(1),:),3,3);
        RJ =reshape(Rs(conn(2),:),3,3);
        x1 = x0+U1(:,1:3); % coordinates of nodes
        [L1,T,d_N]=local_frames(x0,x1x2_vector(:,i),x1,RI,RJ);
        Te=[T Z Z Z; ...
            Z T Z Z;...
            Z Z T Z;...
            Z Z Z T];
        mu=rho*A(i);
        fLI =Omegatilde2(:,2)*spin_eccentricity; % local distributed load components
        fLJ =Omegatilde2(:,2)*spin_eccentricity; % local distributed load components
        FL = [2/3*fLI(1)+1/3*fLJ(1);
            7/10*fLI(2)+3/10*fLJ(2);
            7/10*fLI(3)+3/10*fLJ(3);
            0;
            -(3*fLI(3)+2*fLJ(3))/15*L1/2;
            (3*fLI(2)+2*fLJ(2))/15*L1/2;
            1/3*fLI(1)+2/3*fLJ(1);
            3/10*fLI(2)+7/10*fLJ(2);
            3/10*fLI(3)+7/10*fLJ(3);
            0;
            (2*fLI(3)+3*fLJ(3))/15*L1/2;
            -(3*fLI(2)+2*fLJ(2))/15*L1/2;
            ]*(-mu*L1/2);
        Fe{i} = Te*FL;
    end
    evs  = elevecset(struct('vec',{Fe}, 'eqnums',{eqnums}));
end
