% Constructor of the force intensity class.
%
% function self = force_intensity (varargin)
%
% Constructor of the force intensity class. The physical units are
% force per unit volume, where volume depends on to which manifold 
% the force is applied:
% force/length^3 (when applied to a 3-D solid), 
% force/length^2 (when applied to a surface), 
% force/length^1 (when applied along a curve), 
% or force/length^0 (when applied at a point).
% See discussion of constructors in <a href="matlab:helpwin 'FAESOR/classes/Contents'">classes/Contents</a>. 
% Parameters:
%      magn=vector of magnitudes, dimension must correspond to the
%            dimension of the displacement/geometry fields; may be an array
%            of doubles or a function handle/in-line function with signature
%                     function val=f(xyz)
%            or         
%                     function val=f(xyz,J)
%            where        
%                     xyz = location, at which the force intensity 
%                           is to be evaluated.
%                     J = Jacobian matrix at the location above.
%
function self = force_intensity (varargin)
    class_name='force_intensity';
    parent = classbase;
    if (nargin == 0)
        self.magn = [];
        self.expect_J= true;
        self = class(self,class_name,parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            self = arg;
            return;
        else
            Parameters = varargin{1};
            self.magn = Parameters.magn;
            self.expect_J= true;
            if strcmp(class (self.magn),'inline') || strcmp(class (self.magn),'function_handle')
                for j=1:3
                    try
                        val=feval(self.magn,zeros(1,j),eye(j));
                        self.expect_J= true;
                        break;
                    catch
                        self.expect_J= ~true;
                    end
                end 
            end
            self = class(self,class_name,parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
