% Update material state.  Return the updated material state, and the
% requested quantity (default is the stress).
%   Call as:
%     [out,newms] = update(m, ms, context)
%  where
%     m=material
%     ms = material state
%     context=structure
%        with mandatory fields
%           F= deformation gradient
%        and optional fields
%           output=type of quantity to output, and interpreted by the
%           particular material; [] is returned when the material does not
%           recognize the requested quantity to indicate uninitialized
%           value.  It can be tested with isempty ().
%              output ='Cauchy' - Cauchy stress; this is the default
%                      when output type is not specified.
%              output ='2ndPK' - 2nd Piola-Kirchhoff stress;
%           It is assumed that stress is output in 6-component vector form.
%              output ='strain_energy'
%   The output arguments are
%     out=requested quantity
%     newms=new material state; don't forget that if the update is final
%           the material state newms must be assigned and stored.  Otherwise
%           the material update is lost!
%
function [out, newms] = update_nice (self, ms, context)
    E = self.E;
    nu = self.nu;
    lambda = E * nu / (1 + nu) / (1 - 2*(nu));          % Lame constant #1
    mu     = E / (2 * (1 + nu));                        % shear modulus
    F1 = context.F;
    F1bar =context.Fbar;
    % Finger deformation tensor
    C=F1bar'*F1bar;
    Cinv =inv(C);
    J=det(F1);   
    Jbar  =det(F1bar);   
    sigma = (F1*(mu* (eye(3,3)-Cinv) + lambda *log(Jbar)* Cinv)*F1')/J ;
    if isfield(context,'output')
        switch context.output
            case 'Cauchy'
                out = stress_3x3t_to_6v(self.mater_defor,sigma);
            case'strain_energy'
                C=F1'*F1;
                out = mu/2*(trace(C)-3) - mu*log(J) + lambda/2*(log(J))^2;
            case '2ndPK'
                invF1=inv(F1);
                S = J * invF1*sigma*invF1';
                out= stress_3x3t_to_6v(self.mater,S);
            otherwise
                out = stress_3x3t_to_6v(self.mater,sigma);
        end
    else
        out = stress;
    end
    newms = ms;
    return;
end

