% Constructor  for thermal diffusion properties of materials.
%
% function retobj = property_thermal_diffusion (varargin)
%
% This class represents Thermal diffusion properties of materials.
% See discussion of constructors in <a href="matlab:helpwin 'FAESOR/classes/Contents'">classes/Contents</a>. 
% Parameters:
%     thermal_conductivity=matrix of conductivities; 2x2 for 2-D domains, 3x3
%                  for 3-D domains; the components are given in the local
%                  coordinate frame (feblock decides what that frame is).
%                  The conductivity may be supplied as a function handle, 
%                  that returns the conductivity is a function of temperature. 
%                  For instance kappa=@(T)(0.09+0.004*T)*eye(2);
%     specific_heat=specific heat per unit volume
%                  The specific heat coefficient may be supplied as 
%                  a function handle, that returns the conductivity 
%                  as a function of temperature. 
%                  For instance v=cp_function(T)
%     source=source of heat: function handle or a constant
%
function retobj = property_thermal_diffusion (varargin)
    class_name='property_thermal_diffusion';
    parent=property;
    if (nargin == 0)
        self.thermal_conductivity = 1.0;
        self.specific_heat= 1.0;
        self.heat_source = [];
        retobj = class(self,class_name, parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters =varargin{1};
            self.thermal_conductivity = Parameters.thermal_conductivity;
            if (~strcmp(class(self.thermal_conductivity),'function_handle'))
                if (det(self.thermal_conductivity) < 0)
                    error('Negative determinant of thermal_conductivity!');
                end
            end
            specific_heat= 1.0;
            if isfield(Parameters,'specific_heat')
                specific_heat=Parameters.specific_heat;
            end
            self.specific_heat = specific_heat;
            heat_source = [];
            if isfield(Parameters,'heat_source')
                heat_source=Parameters.heat_source;
            end
            self.heat_source = heat_source;
            retobj = class(self,class_name, parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end
