% Constructor.
% 
% function retobj = property_viscous_fluid(varargin)
% 
% This class represents material properties of viscous (Newtonian) fluid materials.
% Always takes either zero or one argument.  For no arguments, the default
% object is created.  For one argument, the argument may be either an
% object of the same type, in which case the copy is created; or, it may be
% a struct with the following fields representing parameteres.
% Parameters:
% required fields
%     bulk=bulk modulus
%     eta=shear viscosity
% optional fields
%     alpha = coefficient of thermal expansion
%
function retobj = property_viscous_fluid(varargin)
    class_name='property_viscous_fluid';
    if (nargin == 0)
        parent=property;
        self.bulk = 1.0;
        self.eta = 0.0;
        self.zeta = 0.0;
        self.D =[];
        self.D_visc =[];
        self.alpha = 0.0;
        retobj = class(self,class_name, parent);
        return;
    elseif (nargin == 1)
        arg = varargin{1};
        if strcmp(class(arg),class_name) % copy constructor.
            retobj = arg;
            return;
        else
            Parameters =varargin{1};
            parent=property(Parameters);
            self.bulk= Parameters.bulk;
            self.eta = 0;
            self.zeta = 0;
            self.D =[];
            self.D_visc =[];
            if isfield(Parameters,'eta')
                self.eta = Parameters.eta;
                if (self.eta < 0)
                    error('Negative viscosity!');
                end
            end
            if isfield(Parameters,'zeta')
                self.zeta = Parameters.zeta;
                if (self.zeta < 0)
                    error('Negative viscosity!');
                end
            end
            self.alpha = 0.0;
            if isfield(Parameters,'alpha')
                self.alpha= Parameters.alpha;
            end
            % precompute
            bulk = self.bulk;
            m1 = [1 1 1 0 0 0]';
            self.D = bulk * m1 * m1';
            self.D_visc =  self.zeta * m1 * m1' + 2 * self.eta * [2/3*[2 -1 -1; -1 2 -1; -1 -1 2] zeros(3,3); zeros(3,3) eye(3,3) ];
            retobj = class(self,class_name, parent);
            return;
        end
    else
        error('Illegal arguments!');
    end
    return;
end