% Assemble an array of element matrices.
%
% function retobj = assemble (self, ems)
%
% Assemble a cell array of element matrices, that is objects of type elematset,
% into a sparse matrix.
%
function self = assemble (self, ems)
    % First find out if all matrices are symmetric
    all_symmetric=true;
    ne =length(ems);
    for m = 1:ne
        if ~placesymm(ems(m))
            all_symmetric=~true;
            break;
        end
    end
    if (all_symmetric)
        self = assemble_symm(self, ems);% only for symmetric matrices
    else
        self = assemble_unsymm(self, ems);% general but slow
    end
end

function self = assemble1 (self, ems)
    invalid_eqnum = get(self.sysmat, 'invalid_eqnum');
    [neqns,neqns] = size(self.mat);
    A=[];
    if ~self.initialized
        % first collect all connectivity arrays into a rectangular array for easier
        % and faster processing
        ne =length(ems);
        % find the dimension of the largest element matrix 
        %         and the total number of element matrices
        maxnn=0;
        tnem=0;
        for m = 1:ne
            if placesymm(ems(m))
                eqnums = get(ems(m), 'eqnums');
                for j=1:length(eqnums)
                    maxnn=max([maxnn,length(eqnums{j})]);
                end
                tnem =tnem +length(eqnums);
            else
                eqnums_row = get(ems(m), 'eqnums_row');
                for j=1:length(eqnums_row)
                    maxnn=max([maxnn,length(eqnums_row{j})]);
                end
                eqnums_col = get(ems(m), 'eqnums_col');
                for j=1:length(eqnums_col)
                    maxnn=max([maxnn,length(eqnums_col{j})]);
                end
                tnem =tnem +length(eqnums_row);
            end
            
        end
        % allocate to be big enough for the largest matrix
        conns=zeros(tnem,maxnn)+invalid_eqnum;
        inem =1;
        for m = 1:ne
            eqnums=get(ems(m), 'eqnums');
            for j=1:length(eqnums)
                conns(inem,1:length(eqnums{j})) =eqnums{j};
                inem =inem+1;
            end
        end
        % Now replace invalid equation numbers by repeated valid node indexes
        % (maximum in each row, for simplicity): this does not change the sparsity pattern, but
        % simplifies considerably the code
        mv=max(conns,[],2);  % valid equation numbers in each row
        mv(find(mv==invalid_eqnum))=1;  % if there is no valid equation,
        for i=1:maxnn,
            v=conns(:,i);
            dx=find(v==invalid_eqnum);
            v(dx)=mv(dx);
            conns(:,i)=v;
        end
        % Now compute the sparsity pattern, and construct the sparse matrix
        [sppatti, sppattj] =conn_to_sparse(conns,neqns);
        % whatever we initialized A to, it needs to be subtracted later
        % Note: we are using a trick: in order to preserve the sparse pattern, we
        % needs to initialize the matrix to non-zeros in the potentially nonzero
        % entries.  For simplicity, we initialize to 1.0. this needs to be
        % subtracted from the matrix once it is assembled in order not to affect
        % the results.
        A=sparse(sppatti, sppattj, [1.0], neqns, neqns);
        clear sppatti; clear sppattj;
        self.mat=A;
        self.initialized = true;
    end
    % Now loop over at the element matrices and assemble them
    for m = 1:length(ems)
        if placesymm(ems(m))
            eqnums = get(ems(m), 'eqnums');
            mat    = get(ems(m), 'mat');
            for j=1:length(mat)
                emask=eqnums{j}>invalid_eqnum;
                valid_eqnums=eqnums{j}(emask);
                rmat=mat{j};
                self.mat(valid_eqnums,valid_eqnums) = self.mat(valid_eqnums,valid_eqnums) + rmat(emask,emask);
            end
        else
            eqnums_row = get(ems(m), 'eqnums_row');
            eqnums_col = get(ems(m), 'eqnums_col');
            mat    = get(ems(m), 'mat');
            for j=1:length(mat)
                emaskr=eqnums_row{j}>invalid_eqnum;
                emaskc=eqnums_col{j}>invalid_eqnum;
                valid_eqnums_row=eqnums_row{j}(emaskr);
                valid_eqnums_col=eqnums_col{j}(emaskc);
                rmat=mat{j};
                self.mat(valid_eqnums_row,valid_eqnums_col) = self.mat(valid_eqnums_row,valid_eqnums_col) + rmat(emaskr,emaskc);
            end
        end
    end
    % Note: we are using a trick: in order to preserve the sparse pattern, we
    % needs to initialize the matrix to non-zeros in the potentially nonzero
    % entries.  For simplicity, we initialize to 1.0. this needs to be
    % subtracted from the matrix once it is assembled in order not to affect
    % the results.
    if ~isempty(A)
        self.mat=self.mat-A;
    end
end

function [sppatti, sppattj] = conn_to_sparse(conns,neqns)
    ne=size(conns,1);
    nn=size(conns,2);
    rowp = cell(neqns,1);
    for i=1:ne
        c=unique(conns(i,:));
        for j=1:length(c)
            ij=c(j);
            rowp{ij}= [rowp{ij} c];
        end
    end
    sl=0;
    for i=1:neqns
        rowp{i}= unique(rowp{i});
        sl=sl+length(rowp{i});
    end
    sppatti= zeros (1,sl);
    sppattj= zeros (1,sl);
    k=1;
    for i=1:neqns
        r= rowp{i};
        rn=length(r);
        if ~isempty (r)
            sppatti(k:k+rn-1) =i;
            sppattj(k:k+rn-1) =r;
            k = k+rn;
        end
    end
    sppatti=sppatti(1:k-1);
    sppattj=sppattj(1:k-1);
    return;
end

function self = assemble_symm (self, ems)
    invalid_eqnum = get(self.sysmat, 'invalid_eqnum');
    [neqns,neqns] = size(self.mat);

    ne =length(ems);
    sl=0;
    for m = 1:ne
        eqnums=get(ems(m), 'eqnums');
        for j=1:length(eqnums)
            sl =sl+length(eqnums{j})^2;
        end
    end
    sppatti= zeros (1,sl);
    sppattj= zeros (1,sl);
    Ainit= zeros (1,sl);
    triplet =1;
    for m = 1:ne
        eqnums=get(ems(m), 'eqnums');
        mat    = get(ems(m), 'mat');
        for j=1:length(mat)
            emask=eqnums{j}>invalid_eqnum;
            conns=eqnums{j}(emask);
            rmat=mat{j}(emask,emask);
            for r= 1:length(conns)
                for c= 1:length(conns)
                    sppatti(triplet) =conns(r);
                    sppattj(triplet) =conns(c);
                    Ainit(triplet) =rmat(r,c);
                    triplet =triplet +1;
                end
            end
        end
    end
    self.mat=self.mat+sparse(sppatti(1:triplet-1), sppattj(1:triplet-1), Ainit(1:triplet-1), neqns, neqns);
    self.initialized = true;
end


function self = assemble_unsymm (self, ems)
    invalid_eqnum = get(self.sysmat, 'invalid_eqnum');
    [neqns_row,neqns_col] = size(self.mat);

    ne =length(ems);
    sl=0;
    for m = 1:ne
        eqnums_row=get(ems(m), 'eqnums_row');
        eqnums_col=get(ems(m), 'eqnums_col');
        for j=1:length(eqnums_row)
            sl =sl+length(eqnums_row{j})*length(eqnums_col{j});
        end
    end
    sppatti= zeros (1,sl);
    sppattj= zeros (1,sl);
    Ainit= zeros (1,sl);
    triplet =1;
    for m = 1:ne
        eqnums_row=get(ems(m), 'eqnums_row');
        eqnums_col=get(ems(m), 'eqnums_col');
        mat    = get(ems(m), 'mat');
        for j=1:length(mat)
            emask_row=eqnums_row{j}>invalid_eqnum;
            emask_col=eqnums_col{j}>invalid_eqnum;
            conns_row=eqnums_row{j}(emask_row);
            conns_col=eqnums_col{j}(emask_col);
            rmat=mat{j}(emask_row,emask_col);
            for r= 1:length(conns_row)
                for c= 1:length(conns_col)
                    sppatti(triplet) =conns_row(r);
                    sppattj(triplet) =conns_col(c);
                    Ainit(triplet) =rmat(r,c);
                    triplet =triplet +1;
                end
            end
        end
    end
    self.mat=self.mat+sparse(sppatti(1:triplet-1), sppattj(1:triplet-1), Ainit(1:triplet-1), neqns_row,neqns_col);
    self.initialized = true;
end
